# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2019, 2020. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

import time
import numpy as np
import sys
import argparse
import pandas as pd

from cudf import DataFrame

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)
CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
CLI.add_argument(
   "--fit_intercept",
   action = 'store_true',
   default=False
)

args = CLI.parse_args()
t0 = time.time()

# Load data
pdf_trainX = pd.read_csv(args.data_path + "/data/epsilon.X_train.csv", dtype=np.float32, header=None)
pdf_testX  = pd.read_csv(args.data_path + "/data/epsilon.X_test.csv", dtype=np.float32, header=None)
pdf_trainY = pd.read_csv(args.data_path + "/data/epsilon.y_train.csv", dtype=np.float32, header=None)
pdf_testY  = pd.read_csv(args.data_path + "/data/epsilon.y_test.csv", dtype=np.float32, header=None);

print("Data load time (s):  {0:.2f}".format(time.time()-t0))

# Convert pandas to cudf dataframes
cudf_trainX = DataFrame.from_pandas(pdf_trainX)
cudf_trainY = DataFrame.from_pandas(pdf_trainY)

# ndarray data used for training (for sklearn case)
# Converting to C-contiguous ndarray
X_train_ndarray = np.ascontiguousarray(pdf_trainX.values)
y_train_ndarray = np.ascontiguousarray(pdf_trainY.values)

# data used for training
# Create a C-contiguous DeviceNDArray from cuDF
from pai4sk.sml_io import copy_as_gpu_cmatrix
X_train = copy_as_gpu_cmatrix(cudf_trainX)
y_train = copy_as_gpu_cmatrix(cudf_trainY)

#data used for inferencing
# Converting to C-contiguous ndarray
X_test = np.ascontiguousarray(pdf_testX.values)
y_test = np.ascontiguousarray(pdf_testY.values)

num_threads = 256
cpu_gpu = "GPU"

if args.use_balanced_class_weights:
  class_weight = "balanced"
else:
  class_weight = None

# Create a SVM instance from pai4sk explicit call
from pai4sk import SupportVectorMachine
svm = SupportVectorMachine(use_gpu=True, num_threads=num_threads, 
                           class_weight=class_weight,
                           regularizer=2, fit_intercept=args.fit_intercept)

# Training
t0 = time.time()
svm.fit(X_train, y_train)
print("[pai4sk] {0} Training time (s):  {1:.2f}".format("GPU", time.time()-t0))


# Inference
pred = svm.predict(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import accuracy_score
acc_snap = accuracy_score(y_test, pred)
print("[pai4sk] {0} Accuracy: {1:.3f}".format(cpu_gpu, acc_snap))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import hinge_loss
loss_snap = hinge_loss(y_test, pred)
print("[pai4sk] {0} Hinge-loss: {1:.3f}".format(cpu_gpu, loss_snap))

#This is another way of import

# Create a SVM instance from pai4sk implicit call
from pai4sk.svm import LinearSVC as SupportVectorMachine 
svm = SupportVectorMachine(use_gpu=True, num_threads=num_threads,
                           class_weight=class_weight, fit_intercept=args.fit_intercept,
                           loss="hinge", C=0.5, dual=True, tol=0.001)

# Training
t0 = time.time()
svm.fit(X_train, y_train)
print("[pai4sk.svm] {0} Training time (s):  {1:.2f}".format("GPU", time.time()-t0))


# Inference
pred = svm.predict(X_test)


# Evaluate accuracy on test set
from pai4sk.metrics import accuracy_score
acc_snap = accuracy_score(y_test, pred)
print("[pai4sk.svm] {0} Accuracy: {1:.3f}".format(cpu_gpu, acc_snap))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import hinge_loss
loss_snap = hinge_loss(y_test, pred)
print("[pai4sk.svm] {0} Hinge-loss: {1:.3f}".format(cpu_gpu, loss_snap))

# Create a SVM instance from pai4sk explicit call
from pai4sk import SupportVectorMachine
svm = SupportVectorMachine(use_gpu=True, num_threads=num_threads,
                           class_weight=class_weight,
                           regularizer=2, fit_intercept=args.fit_intercept)

# Train using ndarray
t0 = time.time()
svm.fit(X_train_ndarray, y_train_ndarray)
print("[pai4sk] {0} Training time (s):  {1:.2f}".format("GPU", time.time()-t0))


# Inference
pred = svm.predict(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import accuracy_score
acc_snap = accuracy_score(y_test, pred)
print("[pai4sk] {0} Accuracy: {1:.3f}".format(cpu_gpu, acc_snap))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import hinge_loss
loss_snap = hinge_loss(y_test, pred)
print("[pai4sk] {0} Hinge-loss: {1:.3f}".format(cpu_gpu, loss_snap))

if args.skip_sklearn:
    print('Execution using scikit-learn is skipped.')
    exit()

# Import sklearn's SupportVectorMachine from pai4sk module directly
from pai4sk.svm import LinearSVCSklearn as SupportVectorMachine
svm = SupportVectorMachine(fit_intercept=args.fit_intercept, dual=True,
                           loss="hinge", C=0.5, tol=0.001)

# Training time
t0 = time.time()
svm.fit(X_train_ndarray, y_train_ndarray)
print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

pred = svm.predict(X_test)

from pai4sk.metrics import accuracy_score
acc = accuracy_score(y_test, pred)
print("[sklearn] Accuracy: {0:.3f}".format(acc))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
loss = hinge_loss(y_test, pred)
print("[sklearn] Hinge loss: {0:.3f}".format(loss))
