# HIGGS Example

In this example we will train a Logistic Regression model on the HIGGS dataset, using both `scikit-learn` (`sklearn`) and `pai4sk`. 
Note that `scikit-learn` does not provide an MPI implementation. We will however still perform this comparison in order to mainly verify the accuracy metrics.

The HIGGS dataset is avaliable in the [UCI machine learning repository](https://archive.ics.uci.edu/ml/datasets/HIGGS). 

The problem is described as follows:

> This is a classification problem to distinguish between a signal process which produces Higgs bosons and a background process which does not.

The corresponding citation for the dataset is the following paper:

> Baldi, P., P. Sadowski, and D. Whiteson. “Searching for Exotic Particles in High-energy Physics with Deep Learning.” Nature Communications 5 (July 2, 2014).

## Download the Data

Download and decompress the data from the LIBSVM repository:
```bash
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/HIGGS.bz2
bunzip2 HIGGS.bz2
cd ../
```

Now one should see a file named `HIGGS` in the `data/` directory.

## Data Pre-processing
The data is in libsvm format which is not really appropiate since this dataset is 95% dense. Therefore, we perform the following pre-processing steps.

First we split the data into train and test datasets.

Next we convert the newly created datasets to dense representation.

Then we normalize the data. Please note that it is recommended to normalize the input data in order to achieve the maximum performance. 

Next we write the dense data structures to disk as binary numpy files. This helps to achieve fast data loading. 
This format will be used for loading the datasets using `sklearn`. `sklearn` will be used for performance comparison.

Finally we write the train and test datasets in distributed snap format which will be used only for training the `pai4sk` Logistic Regression model. 
This format helps achieve higher performance.

A summary of all the pre-processed files is shown below:
```bash
ls -l data/
HIGGS
HIGGS.X_train.npy
HIGGS.X_test.npy
HIGGS.y_train.npy
HIGGS.y_train.npy
HIGGS.train.snap
HIGGS.test.snap
```

The entire pre-processing code can be run by executing `snaprun python $CONDA_PREFIX/pai4sk/mpi-examples/example-higgs/preprocess-higgs.py`


## Training and Evaluating a Logistic Regression Model

After running the data pre-processing steps above, we are ready to perform our benchmarking: `snap.ml` vs. `sklearn`.

First we train a Logistic Regression model using the IBM `pai4sk` library. 
We provide command line option --use_gpu to run the example for how to train the model using 1) GPUs or 2) CPUs.

We define the data paths for the train and test datasets in distributed snap format.
You need to have access to the data from all the connected machines that will be used (by replicating the data or by providing a distributed file system).

Next we read the datasets from snap format using the loading functions provided by the `pai4sk` library and we measure the data loading time.

After initialization, we call the `fit()` function in order to train the dual Logistic Regression model. We also time the execution of the training process.

Please note that the `pai4sk` library assumes for maximum performance to run 1 MPI process per node.

As a last step we make predictions on the test data and we compute the accuracy and the logistic loss metrics.
The `pai4sk` library provides distributed implementations of the accuracy and logistic loss metrics.

Finally, we train a Logistic Regression model using the `sklearn` library.

## Results

To run on one machine with 1 MPI process, one should run the following command: 
```bash
snaprun python $CONDA_PREFIX/pai4sk/mpi-examples/example-higgs/example-higgs.py
```

To run on 2 machines with 1 MPI process per machine, one should run the following command:
```bash
snaprun -H host1,host2 python $CONDA_PREFIX/pai4sk/mpi-examples/example-higgs/example-higgs.py
```

The user can also run multiple MPI processes per machine, but, in that case, the MPI processes on one machine will share the same hardware resources during training. Namely, if the user trains the model on GPUs, the MPI processes on one machine will share the same GPUs as defined in the `device_ids` parameter. Therefore, we recommend the user to run 1 MPI process per machine in order to achieve the maximum performance.

One should see the following output on a **P9+V100** node when running `snaprun python $CONDA_PREFIX/pai4sk/mpi-examples/example-higgs/example-higgs.py`:

```
[pai4sk] [Data loading] Time = 1.137
[pai4sk] [GPU Training] Accuracy = 0.643
[pai4sk] [GPU Training] LogLoss = 0.637
[pai4sk] [GPU Training] Training time = 3.723

[sklearn] [Data loading] Time = 0.471
[sklearn] Accuracy = 0.643
[sklearn] LogLoss = 0.637
[sklearn] Training time = 63.484
```

One should see the following results on 2 **P9+V100** systems when running `snaprun -H host1,host2 python $CONDA_PREFIX/pai4sk/mpi-examples/example-higgs/example-higgs.py`:
```
[pai4sk] [Data loading] Time = 0.918
[pai4sk] [GPU Training] Accuracy = 0.643
[pai4sk] [GPU Training] LogLoss = 0.637
[pai4sk] [GPU Training] Training time = 3.114

[sklearn] [Data loading] Time = 0.471
[sklearn] Accuracy = 0.643
[sklearn] LogLoss = 0.637
[sklearn] Training time = 63.484
```

Please note that the reported times can vary depending on the system architecture.

&copy; Copyright IBM Corporation 2018, 2019
